#include <stdarg.h>
#include <ctype.h>
#include <avr/io.h>

#include "bendmatrix2.h"
#include "globals.h"
#include "effects.h"
#include "timer.h"




uint8_t get8bitrow(uint16_t mtx1,uint16_t mtx2, uint8_t row){
/*
* Makes a new 8bit number from two 16 bit inputs:
* row selects bits 0..3 or 4..7 or 8..11 or 12..15
* mtx1 is the bottom 4 bits, mtx2 is the top 4 bits
*/
	uint16_t mask;
	mask = 0x000F << (row*4); //row==0 --> 0x000F, row==1 --> 0x00F0, row==2 --> 0x0F00, row==3 --> 0xF000

	return(
		((mtx1 & mask)>>(row*4)) | (((mtx2 & mask)>>(row*4))<<4)
	);

}

uint8_t get8bitcorner(uint16_t mtx1,uint16_t mtx2, uint8_t corner){
/*
* Makes a new 8bit number from two 16 bit inputs:
* corner selects lower or upper half of mtx1 or mtx 2
*/
	switch (corner){
		case (0):
			return (mtx1 & 0x00FF);
			break;
		case (1):
			return ((mtx1 & 0xFF00)>>8);
			break;
		case (2):
			return (mtx2 & 0x00FF);
			break;
		case (3):
			return ((mtx2 & 0xFF00)>>8);
			break;
		default:
			return(0);
			break;
	}	
}

uint8_t get8bitcol(uint16_t mtx1,uint16_t mtx2, uint8_t col){
/*
* Makes a new 8bit number from two 4x4 bit inputs:
* Takes two 4-bit columns
*/

	switch (col){
		case(0):
			return ((mtx1 & 0x0003) | ((mtx1 & 0x0030)>>2) |
			 ((mtx1 & 0x0300)>>4) | ((mtx1 & 0x3000)>>6));
		break;
		case(1):
			return (((mtx1 & 0x000C)>>2) | ((mtx1 & 0x00C0)>>4) |
			 ((mtx1 & 0x0C00)>>6) | ((mtx1 & 0xC000)>>8));
		break;
		case(2):
			return ((mtx2 & 0x0003) | ((mtx2 & 0x0030)>>2) |
			 ((mtx2 & 0x0300)>>4) | ((mtx2 & 0x3000)>>6));
		break;
		case(3):
			return (((mtx2 & 0x000C)>>2) | ((mtx2 & 0x00C0)>>4) |
			 ((mtx2 & 0x0C00)>>6) | ((mtx2 & 0xC000)>>8));
		break;
		default:
			return(0);
		break;
	}

}

uint16_t cols2squares(uint8_t mx[4], short leftright){
	/* reverse transformation of get8bitcol */
	if (!leftright){
		return(
		(mx[0] & 0x03) | ((mx[1] & 0x03)<<2) | 
		((mx[0] & 0x0C) << 2) | ((mx[1] & 0x0C) << 4) | 
		((mx[0] & 0x30) << 4) | ((mx[1] & 0x30) << 6) |
		((mx[0] & 0xC0) << 6) | ((mx[1] & 0xC0) << 8)
		);
	}else{
		return(
		(mx[2] & 0x03) | ((mx[3] & 0x03)<<2) | 
		((mx[2] & 0x0C) << 2) | ((mx[3] & 0x0C) << 4) | 
		((mx[2] & 0x30) << 4) | ((mx[3] & 0x30) << 6) |
		((mx[2] & 0xC0) << 6) | ((mx[3] & 0xC0) << 8)
		);
	}
}


uint16_t rows2squares(uint8_t matrixhalf[4], short leftright){
	/* reverse transformation of get8bitrows */

	if (!leftright)
		return(
			(matrixhalf[0] & 0x0F)
			| ((matrixhalf[1] & 0x0F)<<4)
			| ((matrixhalf[2] & 0x0F)<<8)
			| ((matrixhalf[3] & 0x0F)<<12)
			);
	else
		return(
			((matrixhalf[0] & 0xF0) >> 4)
			| (matrixhalf[1] & 0xF0)
			| ((matrixhalf[2] & 0xF0)<<4)
			| ((matrixhalf[3] & 0xF0)<<8)
		);
}




void apply_knob_mode(void){
	short use_bpm;
	uint8_t matrixhalf[4];
	uint8_t nextmatrixhalf;
	uint8_t i,j,k;
	uint8_t nextbank,nextcur;
	unsigned int ain_a,ain_b;
	uint8_t flag_update_matrix=0,somethingchanged;


	use_bpm=get_use_bpm();

/* divide the matrix into 8-bit sections, depending on knob_layout_mode 
*/

	if(sub_knob_mode==4){ //OFF
		matrix[0]=s_matrix[curbank][cur][0];
		matrix[1]=s_matrix[curbank][cur][1];
	} else {

		for (k=0;k<(MZ*2);k++){
			switch (knob_layout_mode){
				case(0):
					matrixhalf[k]=get8bitcorner(
							s_matrix[curbank][cur][0],
							s_matrix[curbank][cur][1],k);
					break;
				case(2):
					matrixhalf[k]=get8bitcol(
							s_matrix[curbank][cur][0],
							s_matrix[curbank][cur][1],k);
					break;
				case(1):
				default:
					matrixhalf[k]=get8bitrow(
							s_matrix[curbank][cur][0],
							s_matrix[curbank][cur][1],k);
					break;
			}
		}
	



		switch (knob_mode){

			case RANDOM_MODE:
			/*
				if (just_stepped) {
					matrix[0]=s_matrix[curbank][cur][0];
					matrix[1]=s_matrix[curbank][cur][1];
				}

				for(k=0;k<2;k++){

					j=k*2;
					if (ain[j+1]<120) delay=127-ain[j+1];
					else if (ain[j+1]>135) delay=ain[j+1]-128;
					else delay=0;
				
					if (ain[j]<85) time=~(ain[j] << 2) & 0xF0;
					else if (ain[j]>175) time=(ain[j] >> 2) & 0x0F;
					else time=0xFF;

					if (delay){
						if (inttimerreached(&strobe_timer[k],((long)480000)/((long)(delay*use_bpm)))){
							rand_down=do_random();
							matrixhalf[k]=rand_down & (time | (time<<8));
							strobe_timer[k]=0;
						}
					}
				} */

				for (k=0;k<(MZ*2);k++){
					ain_a=ain[k];
					if (ain_a<9) ain_a=0;
					else ain_a=ain_a>>2; //0..64

					matrixhalf[k]=do_randfx(k,ain_a,matrixhalf[k]);
				}
				flag_update_matrix=1;
			break;


			case ARP_MODE:
				flag_update_matrix=0;
				somethingchanged=0;
				for (k=0;k<(MZ*2);k++){
					/**
					ain_a is arp speed
					ain_b is arp direction (0=no go, 1=fwd, 2=bckwd)
					**/
					if (MZ==2) ain_b=ain[k];
					else ain_b=ain[k*2];
		
					if (ain_b<8){
						if (matrixhalf[k]!=0){
							matrixhalf[k]=0;
					//		flag_update_matrix=1;
						}
					} else {
						if (ain_b<120){
							ain_a=15-(ain_b >> 3);//119>>3 is 14, so ain_a is 15(CCW)..1(center)
							ain_b=2;
						} else if(ain_b>134) {
							ain_a=(ain_b >> 3)-16; //135>>3 is 17, so ain_a is 1(center)..15(CW)
							ain_b=1;
						} else {
							ain_a=1;
							ain_b=0;
						}
						if (ain_a>15) ain_a=15;
						if (ain_a<1) ain_a=1;

						matrixhalf[k]=do_arp(k,ain_b,ain_a,matrixhalf[k],&somethingchanged);
						if (somethingchanged) flag_update_matrix=1;
					}
				}
			break;


			case GLOOB_MODE:
				if (ainchanged || just_stepped){
					ainchanged=0;
					for (j=0;j<(MZ*2);j++) {
						if (MZ==2) ain_b=ain[j];
						else ain_b=ain[j*2];
						if (ain_b<9) ain_b=0;
						if (ain_b>250) ain_b=255;

						if (ain_b>(32*7)) k=7;
						else if (ain_b>(32*6)) k=6;
						else if (ain_b>(32*5)) k=5;
						else if (ain_b>(32*4)) k=4;
						else if (ain_b>(32*3)) k=3;
						else if (ain_b>(32*2)) k=2;
						else if (ain_b>(32*1)) k=1;
						else if (ain_b>(32*0)) k=0;

						if (CLEAR) matrixhalf[j]=0;
						switch(sub_knob_mode){
							case (1):
								matrixhalf[j]|=ain_b;
								break;
							case(0):
								if (ain_b) matrixhalf[j]|=BIT(k);
								break;
							case (3):
								matrixhalf[j]^=ain_b;
								break;
							case(2):
								if (ain_b & BIT(7)) {
									for (i=((ain_b>>4) & 7);i<8;i++) matrixhalf[j]|=BIT(i);
								} else {
									for (i=0;i<(ain_b>>4);i++) matrixhalf[j]|=BIT(i);
								}

						}
					}

					flag_update_matrix=1;
				}
			break;




			case STROBE_MODE:
				if (play_mode){
					/*
					calculate the next matrix in the sequence 
					(as of yet it assumes FWD path mode)
					*/
				
				

	//				nextcur=(cur==patch_play_end[curbank]) ? patch_play_start[curbank]:(cur+1);
	//				nextbank=((nextcur==patch_play_start) && bank_mode) ? (curbank+1):curbank;
	//				if (nextbank>bank_play_end && bank_mode)
	//					nextbank=bank_play_start;

					if (path_mode==1)
						get_prevstep(&nextbank, &nextcur);
					else 
						get_nextstep(&nextbank, &nextcur);


					for(k=0;k<(MZ*2);k++){
						switch (knob_layout_mode){
							case(1):
								nextmatrixhalf=get8bitrow(
										s_matrix[nextbank][nextcur][0],
										s_matrix[nextbank][nextcur][1],k);
								break;
							case(0):
								nextmatrixhalf=get8bitcorner(
										s_matrix[nextbank][nextcur][0],
										s_matrix[nextbank][nextcur][1],k);
								break;
							case(2):
								nextmatrixhalf=get8bitcol(
										s_matrix[nextbank][nextcur][0],
										s_matrix[nextbank][nextcur][1],k);
								break;
							default:
								nextmatrixhalf=0;
								break;
						}

						if (MZ==1) {
							ain_b=ain[k*2+1];
							ain_a=ain[k*2];
						} else {
						//	if (knob_layout_mode==0){
						//		ain_b=ain[((k>>1) * 2)+1];
						//		ain_a=ain[(k>>1) * 2];
						//	} else {
								ain_b=127;
								ain_a=ain[k];
						//	}
						}

						matrixhalf[k]=strobe(k, ain_b, ain_a, matrixhalf[k], nextmatrixhalf);
					}
					flag_update_matrix=1;
				}
		
			break;


		}

		if (flag_update_matrix){
			switch(knob_layout_mode){
				case(1):
					matrix[0]=rows2squares(matrixhalf,0);
					matrix[1]=rows2squares(matrixhalf,1);
					break;
				case(0):	
					matrix[0]=matrixhalf[0] | (matrixhalf[1]<<8);
					matrix[1]=matrixhalf[2] | (matrixhalf[3]<<8);
					break;
				case(2):
					matrix[0]=cols2squares(matrixhalf,0);
					matrix[1]=cols2squares(matrixhalf,1);
					break;
			}
		}

	}
}





/********************************

 ****** P U S H  B L O B  *******


inputs horizontal (h_ain) and vertical (v_ain) values from analog joystick
outputs two 4x4 matrices

matrixhalf0:
****	****	----	----	----	----	----
----	****	****	----	----	----	----
----	----	----	----	****	****	----
----	----	----	----	----	****	****

matrixhalf1:
*---	**--	-*--	----	--*-	--**	---*
*---	**--	-*--	----	--*-	--**	---*
*---	**--	-*--	----	--*-	--**	---*
*---	**--	-*--	----	--*-	--**	---*

********************************/

void push_blob(uint16_t *matrixhalf0, uint16_t *matrixhalf1, uint8_t h_ain, uint8_t v_ain){
short t1,t2;
	*matrixhalf0=0;
	*matrixhalf1=0;
//vertical
//is there a better way to make t1=ain_bit_5 XOR ain_bit_6?
//ain65	00	01	10	11
//t2t1	00	01	11	10
	t2=(v_ain & BIT(6))?1:0; 
	t1=((v_ain & BIT(5))?1:0) ^ ((v_ain & BIT(6))?1:0);


	if (v_ain<122){ 	//down
		if (!t1) *matrixhalf0|= (0x0F << 12);
		if (!t2) *matrixhalf0|= (0x0F << 8);
	}else if (v_ain>134){				//up
		if (t1) *matrixhalf0|= (0x0F << 4);
		if (t2) *matrixhalf0|= (0x0F);
	}

//horizontal
	t2=(h_ain & BIT(6))?1:0; 
	t1=((h_ain & BIT(5))?1:0) ^ ((h_ain & BIT(6))?1:0);

	if (h_ain<122){ 	//down
		if (!t1) *matrixhalf1|= BIT(0) | BIT(4) | BIT(8) | BIT(12);
		if (!t2) *matrixhalf1|= BIT(1) | BIT(5) | BIT(9) | BIT(13);
	}else if (h_ain>134){				//up
		if (t1) *matrixhalf1|= BIT(2) | BIT(6) | BIT(10) | BIT(14);
		if (t2) *matrixhalf1|= BIT(3) | BIT(7) | BIT(11) | BIT(15);
	}


}


/********************************

 ********* S T R O B E **********


inputs two analog values: delayain and widthain, which are translated into
delay-until-strobe-pulse (aka phase) and width-of-strobe-pulse.
Also inputs the current matrix and the "next" matrix in the sequence


	If Strobe width is turned below center (<125) then m_strobe=matrix and m_baseline=0
	If Strobe width is centered (125< x <130) then m_strobe and m_baseline=matrix (no changing)
	If Strobe width is above center (>130) then m_baseline=matrix|the-next-matrix and m_strobe=matrix

********************************/
uint8_t strobe(uint8_t k, uint8_t delayain, uint8_t widthain, uint8_t thismatrix, uint8_t nextmatrix){

//	static uint8_t already_strobed[4]={0,0,0,0};
//	static uint8_t ready_to_strobe[4]={0,0,0,0};
//	static int strobe_timer[4]={0,0,0,0};
//this is obnoxious, but there's some error with using arrays instead of distinct variables
//some evidence point that its with already_strobed[]
//perhaps it's the fact that arrays can't be static????
	short already_strobed=0, ready_to_strobe=0;
	static short already_strobed0,already_strobed1,already_strobed2,already_strobed3;
	static short ready_to_strobe0,ready_to_strobe1,ready_to_strobe2,ready_to_strobe3;
	static int strobe_timer0=0,strobe_timer1=0,strobe_timer2=0,strobe_timer3=0;
	uint8_t m_strobe=0,m_baseline=0,m_out;
	int use_bpm;
	unsigned int strobe_phase=0,strobe_width;
	uint8_t timerstepped=0;

	use_bpm=get_use_bpm();


	if (just_stepped){
		already_strobed0=0;already_strobed1=0;already_strobed2=0;already_strobed3=0;
		strobe_timer0=0;strobe_timer1=0;strobe_timer2=0;strobe_timer3=0;
		ready_to_strobe0=0;ready_to_strobe1=0;ready_to_strobe2=0;ready_to_strobe3=0;
	}
	
	if (widthain<8){
		strobe_width=0;
		strobe_phase=0;
		m_strobe=0;
		m_baseline=0;
	}
	else 
	if (widthain<120) {
		strobe_width=widthain+1; //1 to 120

		strobe_phase=((delayain+1)*(125-widthain))>>8;

		m_strobe=thismatrix;
		m_baseline=0;

		switch(sub_knob_mode){
			case (0):
			case (2):
				m_strobe=thismatrix;
				m_baseline=0;
				break;
	 		case (1):
			case (3):
				m_strobe=thismatrix;
				m_baseline=0;
				break;
		}
	}
	else if (widthain>135) {
		strobe_width=widthain-135; //1 to 120

		strobe_phase=((delayain+1)*(strobe_width))/256; //1..256 * 0..119 / 256

		switch(sub_knob_mode){
			case (1):
				m_strobe=nextmatrix|thismatrix;
				m_baseline=thismatrix;
				break;
	 		case (0):
				m_strobe=thismatrix;
				m_baseline=thismatrix|nextmatrix;
				break;
		 	case (2):
				m_strobe=thismatrix;
				m_baseline=nextmatrix;
				break;
			case (3):
				m_strobe=nextmatrix;
				m_baseline=thismatrix|nextmatrix;
				break;
		}

	}
	else {
		strobe_width=0;
//		strobe_phase=0;
		m_strobe=thismatrix;
		m_baseline=thismatrix;
	}


	strobe_width=strobe_width*480; //strobe_width ranges 0..125 * 480 = 0.. 60000
	strobe_phase=strobe_phase*480;
	
	m_out=m_baseline;
	
	if (k==0) ready_to_strobe=ready_to_strobe0;
	else if (k==1) ready_to_strobe=ready_to_strobe1;
	else if (k==2) ready_to_strobe=ready_to_strobe2;
	else if (k==3) ready_to_strobe=ready_to_strobe3;

	if (!ready_to_strobe){
		if (k==0) timerstepped=inttimerreached(&strobe_timer0,strobe_phase/use_bpm);
		else if (k==1) timerstepped=inttimerreached(&strobe_timer1,strobe_phase/use_bpm);
		else if (k==2) timerstepped=inttimerreached(&strobe_timer2,strobe_phase/use_bpm);
		else if (k==3) timerstepped=inttimerreached(&strobe_timer3,strobe_phase/use_bpm);
		if (timerstepped){
			if (k==0) {strobe_timer0=0;ready_to_strobe0=1;}
			else if (k==1) {strobe_timer1=0;ready_to_strobe1=1;}
			else if (k==2) {strobe_timer2=0;ready_to_strobe2=1;}
			else if (k==3) {strobe_timer3=0;ready_to_strobe3=1;}
		}
	}
	else
	{
		if (k==0) already_strobed=already_strobed0;
		else if (k==1) already_strobed=already_strobed1;
		else if (k==2) already_strobed=already_strobed2;
		else if (k==3) already_strobed=already_strobed3;

		if (!already_strobed){

			if (k==0) timerstepped=inttimerreached(&strobe_timer0,strobe_width/use_bpm);
			else if (k==1) timerstepped=inttimerreached(&strobe_timer1,strobe_width/use_bpm);
			else if (k==2) timerstepped=inttimerreached(&strobe_timer2,strobe_width/use_bpm);
			else if (k==3) timerstepped=inttimerreached(&strobe_timer3,strobe_width/use_bpm);
			if (timerstepped){
				if (k==0) already_strobed0=1;
				else if (k==1) already_strobed1=1;
				else if (k==2) already_strobed2=1;
				else if (k==3) already_strobed3=1;
			} else {
				m_out=m_strobe;

			}
		}
	}

	return (m_out);
}


/*
		if (knob_layout_mode==0){
				for (k=0;k<2;k++) {

					push_blob(&tmatrixhalf[0],&tmatrixhalf[1],ain[k*2+1],ain[k*2]);

					switch(sub_knob_mode){
						case (0):
							matrix[k]=s_matrix[curbank][cur][k] | ain[k*2] | (ain[k*2+1] <<8);
							break;
						case (1):
							if (tmatrixhalf[0] && tmatrixhalf[1]) matrix[k]=tmatrixhalf[0] & tmatrixhalf[1];
							else if (tmatrixhalf[0]) matrix[k]=tmatrixhalf[0];
							else if (tmatrixhalf[1]) matrix[k]=tmatrixhalf[1];
							matrix[k]|=s_matrix[curbank][cur][k];
							break;
						case (2):
							matrix[k]=s_matrix[curbank][cur][k] | (tmatrixhalf[0] & tmatrixhalf[1]);
							break;
						case (3):
							matrix[k]=s_matrix[curbank][cur][k] | (tmatrixhalf[0] ^ tmatrixhalf[1]);
							break;
					}
				}
			}
*/


inline unsigned short do_random(void)
{
	return (TCNT1);
}




uint8_t do_arp(uint8_t k, uint8_t directionain, uint8_t speedain, uint8_t matrixhalf, uint8_t *somethingchanged){
	short use_bpm;
	uint8_t t;
	uint8_t timerstepped;
	int timebase;
	short arp_i=0,old_arp_i=0;
	static int arp_timer0=0,arp_timer1=0,arp_timer2=0,arp_timer3=0;
	static short arp_i0=0,arp_i1=0,arp_i2=0,arp_i3=0;
	

//need to do some "newspeedain" so the speed timer doesnt crap out when you turn the knob

	use_bpm=get_use_bpm();

	timebase=15000/(use_bpm*speedain);
	if (speedain<15)  timebase=timebase*4; //gets around having to use 60000 which is not a short integer

	if (k==0) 	timerstepped=inttimerreached(&arp_timer0,timebase);
	else if (k==1) 	timerstepped=inttimerreached(&arp_timer1,timebase);
	else if (k==2) 	timerstepped=inttimerreached(&arp_timer2,timebase);
	else 			timerstepped=inttimerreached(&arp_timer3,timebase);

	if (k==0) arp_i=arp_i0;
	else if (k==1) arp_i=arp_i1;
	else if (k==2) arp_i=arp_i2;
	else if (k==3) arp_i=arp_i3;
			

	*somethingchanged=0;
	if (timerstepped || just_stepped){

		/** 
		When the arp_timer steps (or we changed patches)
		look for the next note to play
		if we're in single mode then go to the end and stop
		if we're in repeat arp mode, then cycle the whole thing
		**/
		if (k==0) arp_timer0=0;
		else if (k==1) arp_timer1=0;
		else if (k==2) arp_timer2=0;
		else if (k==3) arp_timer3=0;

		if (!matrixhalf){ 
			/* no point in arpeggiating blank data! */
			arp_i=99;
		}else{
old_arp_i=arp_i;
			/* init the arp pointer if needed*/
			if (directionain==0) arp_i=-1; //find first note only when knob is centered
			else if (just_stepped && (directionain==1)) arp_i=-1;
			else if (just_stepped && (directionain==2)) arp_i=8;

			t=0;
			do{

				if (directionain==2) arp_i--;
				else arp_i++;
				
				switch (sub_knob_mode){
					case (0):	//repeat arp
						arp_i=arp_i & 7;
						break;
					case (1):	//one-shot arp and hold
						if (arp_i != (arp_i&7)) {
							if (directionain==1) {directionain=2;arp_i=7;}
							else if (directionain==2) {directionain=1;arp_i=0;}
						} //if we overflow, go to the last bit and count backwards
						break;
					case (2):	//one-shot arp and off
						if (arp_i != (arp_i&7)) arp_i=99; //if we overflow, flag arp_i to exit the loop
						break;
					case (3):	//random arp
						arp_i=do_random() & 7;
						break;
				}
				t++;
			}while ( (t<16) && (arp_i!=99) && !(matrixhalf & BIT(arp_i)) ) ;
		}
		if (old_arp_i != arp_i) *somethingchanged=1;

		if (k==0) arp_i0=arp_i;
		else if (k==1) arp_i1=arp_i;
		else if (k==2) arp_i2=arp_i;
		else if (k==3) arp_i3=arp_i;

	} //(timerstepped || just_stepped)

	if (arp_i!=99)
		return(BIT(arp_i));
	else 
		return(0);

}

uint8_t do_randfx(uint8_t k, uint8_t speedain, uint8_t matrixhalf){
	short use_bpm;

	uint8_t timerstepped,i;
	uint8_t rand;
	static int timer0=0,timer1=0,timer2=0,timer3=0;
	static uint8_t lastmatrix0=0,lastmatrix1=0,lastmatrix2=0,lastmatrix3=0;

	use_bpm=get_use_bpm();


	if (speedain>0){
		if (k==0) matrixhalf=lastmatrix0;
		else if (k==1) matrixhalf=lastmatrix1;
		else if (k==2) matrixhalf=lastmatrix2;
		else if (k==3) matrixhalf=lastmatrix3;

		if (k==0) 	timerstepped=inttimerreached(&timer0,60000/(use_bpm*speedain));
		else if (k==1) 	timerstepped=inttimerreached(&timer1,60000/(use_bpm*speedain));
		else if (k==2) 	timerstepped=inttimerreached(&timer2,60000/(use_bpm*speedain));
		else 			timerstepped=inttimerreached(&timer3,60000/(use_bpm*speedain));

		if (timerstepped || just_stepped){
				 if (k==0) timer0=0;
			else if (k==1) timer1=0;
			else if (k==2) timer2=0;
			else if (k==3) timer3=0;

			for (i=0;i<(sub_knob_mode+1);i++){
				rand=do_random() & 0b111;
				matrixhalf^=BIT(rand);
			}

			if (k==0) lastmatrix0=matrixhalf;
			else if (k==1) lastmatrix1=matrixhalf;
			else if (k==2) lastmatrix2=matrixhalf;
			else if (k==3) lastmatrix3=matrixhalf;

		}
	}
	return(matrixhalf);
}


