#include <stdarg.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/eeprom.h>

#include "bendmatrix2.h"
#include "globals.h"
#include "inputs.h"
#include "outputs.h"
#include "midi.h"
#include "memory.h"

unsigned char AIN[4]={AIN0,AIN1,AIN2,AIN3};	//analog input pin assignments


void wait_for_ADC(void){	while( !(ADCSRA & BIT(ADIF)) );	}

void read_adc(void){

	static unsigned char old_ain[4]={0,0,0,0};
	short i,j,k;
	
	for (i=0;i<4;i++){
		ADMUX=AIN[i] | BIT(ADLAR); 
  	    ADCSRA |= BIT(ADSC);		//set the Start Conversion Flag in the ADC Status Register
		wait_for_ADC();
    	ADCSRA |= BIT(ADIF);		// Clear the flag by sending a logical "1"
  
		j=ADCH;
		k=j-old_ain[i];
		if (k<0) k=-k;

		if (k>7){
			ainchanged=i+1;		//flag for GLOOB mode to know to update the matrix
			ain[i]=j;
			old_ain[i]=ain[i];

			#ifdef SENDMIDICC
				send_MIDI_CC(i,ain[i]>>1);
			#endif

			if (assign_midi_mode || (SHIFT && (hold_menu_display==MIDI_MODE))){
				if (assign_midi_mode!=2) show_midi_menu_display=1;
				assign_midi_mode=2;
				assign_midi_curbut=i;
			}
		}
	}

}

void load_input_shiftreg(void){

	SR &= ~BIT(SR_PL); 		//load parallel data
	SR |= BIT(SR_PL); 		//switch to serial shift mode
}

void read_buttonmatrix(void){
	unsigned char i,k,kk,k2,k3;
	short bank;
	static uint16_t matrix_down[2]={0,0};
/*
	read matrix's button data from shift register
	and do appropriate action (toggle bit in matrix[], or save/recall bank

	This is a hack, but it works:

	with MZ==2:
	k =0,1,0,1 (red, green, red, green)
	k2=0,0,1,1 solely is used to generate i
	i =0..7,0..7,8-15,8-15 (low MX, high MX, low MX, high MX)

	with MZ==1:
	k = 0,1
	i = 0..7,0..7
*/

   for (k=0,k2=0,k3=0;k3<(2*MZ);k3++,k2+=k,k=!k){
		for(i=k2*8;i<(8*k2+8);i++){

			if (MZ==2){
				if (k) bank=-1;
				else
				 bank=i;
			} else {
				bank=i+(k*8);
			}
			
			if (SR_serial && save_bank_mode && (bank!=-1)){
				save_patch(curbank,cur,switch_matrix);

				save_bank(bank,curbank);

				save_bank_mode=0;
				curbank=bank;
				matrix_down[k]|=BIT(i);

			
			} else if (assign_midi_mode || (SHIFT && (hold_menu_display==MIDI_MODE))){
				if (SR_serial)
					buttonmatrix[k]|=BIT(i);
				else 
					buttonmatrix[k]&=~BIT(i);

				if (SR_serial && !(matrix_down[k] & BIT(i))){
					matrix_down[k]|=BIT(i);

					if (SHIFT){
						show_midi_menu_display=1;
						#ifdef MAP_CCS_TO_BUTTONS
							if (assign_midi_mode==1) assign_midi_mode=3;
							else assign_midi_mode=1;
						#else
							assign_midi_mode=1;
						#endif
					}

					if ((assign_midi_mode==0) || (assign_midi_mode==2)) {
						assign_midi_mode=1;
						show_midi_menu_display=1;
					}

					assign_midi_curbut=i+(k*16);
					length_mode=0; //turn length_mode off before we go into midi_assign mode!!
				}
				else if (!SR_serial)	matrix_down[k]&=~BIT(i);


			} else if (SR_serial && SHIFT && (bank!=-1)){
				//bank recall
				for (kk=0;kk<2;kk++) matrix[kk]=s_matrix[bank][cur][kk];
				
				curbank=bank;
				number_to_string(bank+1,seven_seg);
				seven_seg[0]=TOPO;
				seven_seg_key=1;
				matrix_down[k]|=BIT(i);

				just_stepped=1;


			} else if (SR_serial && length_mode && (bank!=-1)){
				if (length_mode==5){
					bank_play_end=bank;
					if (bank_play_end<bank_play_start)
						bank_play_start=bank_play_end;
					
					save_bank_length();
					number_to_string(bank+1,seven_seg);
					seven_seg[0]=DASH;
					seven_seg_key=1;
					length_mode=0;
					matrix_down[k]|=BIT(i);
				}

				if (length_mode==1)	{
					bank_play_start=bank;
					if (bank_play_end<bank_play_start)
						bank_play_end=bank_play_start;

					number_to_string(bank+1,seven_seg);
					seven_seg[0]=seven_seg[1];
					seven_seg[1]=seven_seg[2];
					seven_seg[2]=DASH;
					seven_seg_key=1;
					length_mode=4;
					matrix_down[k]|=BIT(i);
				}

			
			} else if (CLEAR){ 
				if (SR_serial && !(matrix_down[k] & BIT(i))){
					matrix_down[k]|=BIT(i);	

					if (latchoff_matrix[k] & BIT(i)){  //on CLEAR+button, toggle the bit in latchoff matrix
						latchoff_matrix[k] &= ~BIT(i);
					} else {
						latchoff_matrix[k] |= BIT(i);	//when it goes ON in the latchoff layer...
						latchon_matrix[k] &= ~BIT(i);	//...we must turn it OFF in the latchon layer
					}
				}
				else if (!SR_serial)	matrix_down[k]&=~BIT(i);


			} else if (SAVE){ 
				if (SR_serial && !(matrix_down[k] & BIT(i))){
					save_mode=0;
					matrix_down[k]|=BIT(i);	

					if (latchon_matrix[k] & BIT(i)){  //on SAVE+button, toggle the bit in latchon matrix
						latchon_matrix[k] &= ~BIT(i);
					} else {
						latchon_matrix[k] |= BIT(i);	//when it goes ON in the latchon layer...
						latchoff_matrix[k] &= ~BIT(i);	//...we must turn it OFF in the latchoff layer
					}
				}
				else if (!SR_serial)	matrix_down[k]&=~BIT(i);
			

			} else if (MOMENTMODE || play_mode ){
				if (SR_serial) {
					#ifdef ONE_PER_ROW_GREEN
						if (k==1) buttonmatrix[k]&=~(0b1111 << (i & 12)); //mask out the row: all off
					#endif						
					
					buttonmatrix[k] |= BIT(i);

					if (matrix[k] & BIT(i)) matrix[k] &= ~BIT(i);

				} else {
					buttonmatrix[k] &= ~BIT(i);
				}

			} else if (SR_serial && !(matrix_down[k] & BIT(i)) ) { 
				//if hit button and havent already set the matrix_down flag...

				if (switch_matrix[k] & BIT(i))
				{	
					buttonmatrix[k]&=~BIT(i); 
					matrix[k]&=~BIT(i);
					midimatrix[k]&=~BIT(i);
				}
				else {
					#ifdef ONE_PER_ROW_GREEN
						if (k==1) buttonmatrix[k] &= ~(0b1111 << (i & 12)); //mask out the row: all off
					#endif						
					
					buttonmatrix[k] |= BIT(i);
				}
						//...flag it in matrix_down so we don't keep flipping the switch
				matrix_down[k]|=BIT(i);	


						//if the button goes off, reset the flag
			} else if (!SR_serial)	matrix_down[k] &= ~BIT(i);


			SR &= ~BIT(SR_CP); //cycle clock low then high
			SR |= BIT(SR_CP);  //to shift the next serial bit in the the AVR

		}
	}

	if ((matrix_down[0]==0) && (matrix_down[1]==0) && (length_mode==4)) length_mode=5;
}


unsigned short read_seqs(unsigned short seq_leds)
{
	unsigned short i,shift_hit=99,k;
	static unsigned short seq_down=0;	

//	seq_down=0;
	for(i=0;i<8;i++){

		if (SR_serial) {			//if sequencer button number "i" is down
			seq_leds = BIT(i);


			
			if (SHIFT){
				shift_hit=i;
			}else{

				//set patch play length
				if (length_mode) {
					if (length_mode==3){
						patch_play_end[curbank]=i;
						if (patch_play_end[curbank]<patch_play_start[curbank])
							patch_play_start[curbank]=patch_play_end[curbank];
						save_patch_length(curbank);
						number_to_string(i+1,seven_seg);
						seven_seg[0]=DASH;
						seven_seg_key=1;
						length_mode=0;
					}

					if ((length_mode==1) && !(seq_down & BIT(i))){
						patch_play_start[curbank]=i;
						if (patch_play_end[curbank]<patch_play_start[curbank])
							patch_play_end[curbank]=patch_play_start[curbank];
						number_to_string(i+1,seven_seg);
						seven_seg[0]=seven_seg[1];
						seven_seg[1]=seven_seg[2];
						seven_seg[2]=DASH;
						seven_seg_key=1;
						length_mode=2;
					}

				}

				//save patch
				else if (save_mode){
					save_mode=0;
					save_patch(curbank,i,switch_matrix);
				}
				
				//recall patch
				else{ 
					for (k=0;k<2;k++) {
						if (knob_mode!=ARP_MODE) matrix[k]=s_matrix[curbank][i][k];
						buttonmatrix[k]=0;
						midimatrix[k]=0;
					}
					PLAYSAVE_LED &= ~BIT(SAVE_LED);

					number_to_string(i+1,seven_seg);
					seven_seg[0]='O'-ALPHA;

					just_stepped=1;
				}
				cur=i;
			}

				seq_down|=BIT(i);
		} else 	seq_down &= ~BIT(i);



		SR &= ~BIT(SR_CP); //cycle clock low then high
		SR |= BIT(SR_CP);  //to shift the next serial bit in the the AVR
	}

	if ((length_mode==2) && (seq_down==0)) length_mode=3;

	change_mode(shift_hit);

	return (seq_leds);
}





void read_trigs(void)
{
	unsigned short i;
	static unsigned short trig_down=0;	
	uint8_t nextbank, nextcur;
	static short startupdonttrigger=200;

	if (startupdonttrigger>0) startupdonttrigger--;
	else {

		for(i=0;i<8;i++){

			if (SR_serial) {			//if trigger is high

				if (!(trig_down & BIT(i))){ //if it's not already triggered
					trig_down|=BIT(i);
				
					if (i==5){
						//STEP FWD
						get_nextstep(&nextbank, &nextcur);
						curbank=nextbank;
						cur=nextcur;
						number_to_string(curbank+1,seven_seg);
						just_stepped=1;
						seq_leds=BIT(cur);
						matrix[0]=s_matrix[curbank][cur][0];
						matrix[1]=s_matrix[curbank][cur][1];


					} else if (i==6){
						//STEP BACK
						get_prevstep(&nextbank, &nextcur);
						curbank=nextbank;
						cur=nextcur;
						number_to_string(curbank+1,seven_seg);
						just_stepped=1;
						seq_leds=BIT(cur);
						matrix[0]=s_matrix[curbank][cur][0];
						matrix[1]=s_matrix[curbank][cur][1];


					} else if (i==7){
						//PLAYPAUSE
						trig_playpause=1;
					}

				}
			} else 
				trig_down &= ~BIT(i);


			SR &= ~BIT(SR_CP); //cycle clock low then high
			SR |= BIT(SR_CP);  //to shift the next serial bit in the the AVR
		}

	}
}



void change_mode(unsigned short i){

static short knob_mode_down=0;
static short knob_layout_mode_down=0;
static short length_mode_down=0;
static short path_mode_down=0;
static short midi_mode_down=0;

	switch (i){
		case (STROBE_MODE):
		case (ARP_MODE):
		case (GLOOB_MODE):
		case (RANDOM_MODE):
			if (!knob_mode_down) {
				hold_menu_display=i;
				knob_mode_down=1;

				if (knob_mode==i) {
					sub_knob_mode++;
					if (sub_knob_mode>4) sub_knob_mode=0;
				} else {
					sub_knob_mode=0;
					knob_mode=i;
				}

				if (sub_knob_mode<4) {
					seven_seg[0]=KNOBMODENAMES[knob_mode];
					seven_seg[1]=BLANK;
					seven_seg[2]=sub_knob_mode+1;
				} else {
					seven_seg[0]=DASH;
					seven_seg[1]=DASH;
					seven_seg[2]=DASH;}
				seven_seg_key=1;
			}
			break;


		case (PATH_MODE):
			if (path_mode_down==0) {
				hold_menu_display=i;
				if (++path_mode>3) path_mode=0;
				path_mode_down=1;
			}
			string_to_string(REPEATNAMES[path_mode], seven_seg);
			seven_seg_key=1;
			break;

		case (LENGTH_MODE): //set sequence length
			assign_midi_mode=0; //assign_midi_mode and length_mode both use the button matrix for input
									///..so we don't want them active at the same time
			if (length_mode_down==0) {
				length_mode=1;
				length_mode_down=1;
			}
			break;

		case (KNOBLAYOUT_MODE):
			if (knob_layout_mode_down==0) {
				hold_menu_display=i;
				if (++knob_layout_mode>=3) knob_layout_mode=0;
				knob_layout_mode_down=1;
			}
			string_to_string(KLNAMES[knob_layout_mode], seven_seg);
			seven_seg_key=1;
//			save_knob_layout_mode();
			break;

		case (MIDI_MODE):
			if (midi_mode_down==0){
				if (hold_menu_display==MIDI_MODE){ //On the initial SHIFT+MIDI, don't increment the mode
					if (++midi_mode>=8) midi_mode=0;	//...this way, it just displays the current mode
						#ifdef MIDI_OFF					
							midi_mode=3;
						#endif
				}

				hold_menu_display=i;
				midi_mode_down=1;
			}
			if (!assign_midi_mode){ //don't display the midi mode if we're assigning
				string_to_string(MIDIMODENAMES[midi_mode], seven_seg);
				seven_seg_key=1;
			}
			break;
	
		case (99): //none down
			knob_layout_mode_down=0;
			midi_mode_down=0;
			path_mode_down=0;
			length_mode_down=0;
			knob_mode_down=0;
			break;
	}

	if (!SHIFT) {
		if (hold_menu_display==MIDI_MODE) save_midi_mode();
		hold_menu_display=0;
	}

}



void check_save(void){
	static char save_down=0;

	if (SAVE){
		length_mode=0;

		if (!save_down){
			PLAYSAVE_LED |= BIT(SAVE_LED);
			save_down=1;

			if ((assign_midi_mode==1) || (assign_midi_mode==2)|| (assign_midi_mode==3)) {
				assign_midi_mode=99;

			} else if (SHIFT){
				//save bank mode
				if (save_bank_mode) save_bank_mode=0;
				else save_bank_mode=1;

			}else{
				if (save_bank_mode){
					save_bank_mode=0;
				}else{
					//save sequence mode
					if (save_mode){
						save_patch(curbank,cur,switch_matrix);
						save_mode=0;
						if (++cur>patch_play_end[curbank]) cur=patch_play_start[curbank];
						matrix[0]=s_matrix[curbank][cur][0];buttonmatrix[0]=0;
						matrix[1]=s_matrix[curbank][cur][1];buttonmatrix[1]=0;
					}
					else save_mode=1;
				}
			}
		} //else {
		//	save_down++;
		//	if (save_down>200) save_mode=0;

	//	}

	}else{ // !SAVE
		if (save_down){
			PLAYSAVE_LED &= ~BIT(SAVE_LED);
			save_down=0;
		}
	}

}



void check_play(void){
	
	static char play_down=0;

	if (PLAY || trig_playpause){
		length_mode=0;

		if (!play_down) {
			
			play_down=1;
			trig_playpause=0;
			
			if (!play_mode) {
				play_mode=1;play_timer=0;flash_timer=0;
				if (curbank<bank_play_start) curbank=bank_play_start;
				if (curbank>bank_play_end) curbank=bank_play_end;
				if (cur<patch_play_start[curbank]) cur=patch_play_start[curbank];
				if (cur>patch_play_end[curbank]) cur=patch_play_end[curbank];

				if (path_mode==1) {
					if (cur==patch_play_end[curbank]) {
						if (bank_mode && (curbank==bank_play_end)) curbank=bank_play_start;
						cur=patch_play_start[curbank];
					}
				}
				timerflash=1; PLAYSAVE_LED |= BIT(PLAY_LED);	
				send_MIDI_command(BPM_START_MIDI,0x00,0x00);
			} else if (!SHIFT) {
				play_mode=0;
				send_MIDI_command(BPM_STOP_MIDI,0x00,0x00);
			}

			if (SHIFT) bank_mode=!bank_mode; 
		
		}
	}else{
		if (play_down)	play_down=0;
	}
	
}



void check_clear(void){
	static int clear_down=0;
	short i,k;	

	if (!CLEAR){ 				//if CLEAR button is down..
		clear_down=0;
	} else {
		

		if (assign_midi_mode || save_bank_mode || length_mode || save_mode) {
			save_mode=0;
			save_bank_mode=0;
			length_mode=0;
			assign_midi_mode=0;
			seven_seg_key=DISPLAYHOLD;
		} else {

//This code turns all buttons ON if you hit clear when they're all off
//			if (switch_matrix[0] || switch_matrix[1])		
//				clearingfilling=0;
//
//			if (clearingfilling){
//				buttonmatrix[0]=~0;
//				buttonmatrix[1]=~0;
//			} else {
//				buttonmatrix[0]=0;buttonmatrix[1]=0;
//				midimatrix[0]=0;midimatrix[1]=0;
//				matrix[0]=0;matrix[1]=0;
//			}
//End this code

			if ((!play_mode && !clear_down) || (SHIFT && play_mode)){
				buttonmatrix[0]=0;buttonmatrix[1]=0;
				midimatrix[0]=0;midimatrix[1]=0;
				matrix[0]=0;matrix[1]=0;
			}

			if (SHIFT && !play_mode){
				for (k=0;k<2;k++){
					for(i=0;i<8;i++) s_matrix[curbank][i][k]=0;	//clear all eight steps
					matrix[k]=0;
					buttonmatrix[k]=0;
					midimatrix[k]=0;
				}
			}

		} //if save_mode

		clear_down=1;

	}	//if CLEAR
}
