#include <stdarg.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/eeprom.h>

#include "bendmatrix2.h"
#include "globals.h"
#include "outputs.h"
#include "memory.h"


void save_midi_assigns(void){
	short i;

	for (i=0;i<16*MZ;i++){
		eeprom_busy_wait();
		eeprom_write_byte ((uint8_t *)(EEPROM_MIDI_NOTE_ASSIGN_ADDR+i),assigned_midi_notes[i]);

		#ifdef MAP_CCS_TO_BUTTONS
			eeprom_busy_wait();
			eeprom_write_byte ((uint8_t *)(EEPROM_MIDI_CCBUTTON_ASSIGN_ADDR+i),assigned_midi_CCbuttons[i]);
		#endif
	}

	for (i=0;i<4;i++){
		eeprom_busy_wait();
		eeprom_write_byte ((uint8_t *)(EEPROM_MIDI_CC_ASSIGN_ADDR+i),CC_number[i]);
	}

}

void save_bank(uint8_t dest_bank, uint8_t src_bank){
	short j,kk;

	eeprom_busy_wait();
	eeprom_write_block(&s_matrix[src_bank],(void *)(dest_bank*4*8),4*8); //32 bits or 4Bytes * 8 patches	

	if (src_bank!=dest_bank){ 	//if we saved into a different bank, 
		for (j=0;j<8;j++) {
			for(kk=0;kk<2;kk++)
				s_matrix[dest_bank][j][kk]=s_matrix[src_bank][j][kk];
		}
	}

			
//	eeprom_busy_wait();
//	eeprom_write_word((uint16_t *)(EEPROM_BPM_ADDR+(dest_bank*2)),bpm[src_bank]); 

}



void save_patch(uint8_t dest_bank, uint8_t dest_patch, uint16_t dest_matrix[2]){
	s_matrix[dest_bank][dest_patch][0]=dest_matrix[0];
	s_matrix[dest_bank][dest_patch][1]=dest_matrix[1];
	PLAYSAVE_LED &= ~BIT(SAVE_LED);

	number_to_string(dest_patch+1,seven_seg);
	seven_seg[0]='O'-ALPHA;

/*
	eeprom_busy_wait();
	eeprom_write_block(&(s_matrix[dest_bank]),(void *)(dest_bank*2*32),8*2*2);
	eeprom_busy_wait();
*/
}


void save_dualmatrix_mode(void){
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_GLOBALS_ADDR+5),DUALMATRIXMODE);
	eeprom_busy_wait();
}

void save_midi_mode(void){
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_GLOBALS_ADDR+4),midi_mode);
	eeprom_busy_wait();
}

void save_knob_mode(void){
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_GLOBALS_ADDR),knob_mode);

	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_GLOBALS_ADDR+1),sub_knob_mode);
	eeprom_busy_wait();
}

void save_path_mode(void){
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_GLOBALS_ADDR+2),path_mode);
	eeprom_busy_wait();
}

void save_knob_layout_mode(void){
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_GLOBALS_ADDR+3),knob_layout_mode);
	eeprom_busy_wait();
}

void save_patch_length(uint8_t src_bank){
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_LENGTH_ADDR+src_bank),patch_play_start[src_bank]);
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_LENGTH_ADDR+16+src_bank),patch_play_end[src_bank]);
}

void save_bank_length(void){
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_LENGTH_ADDR+32),bank_play_start);
	eeprom_busy_wait();
	eeprom_write_byte ((uint8_t *)(EEPROM_LENGTH_ADDR+33),bank_play_end);
	eeprom_busy_wait();

}

void read_midi_assigns(void){

	eeprom_read_block(
		(void *)&(assigned_midi_notes),(const void *)EEPROM_MIDI_NOTE_ASSIGN_ADDR,32
	);

	
	#ifdef MAP_CCS_TO_BUTTONS
		eeprom_read_block(
			(void *)&(assigned_midi_CCbuttons),(const void *)EEPROM_MIDI_CCBUTTON_ASSIGN_ADDR,32
		);
	#endif

	eeprom_read_block(
		(void *)&(CC_number),(const void *)EEPROM_MIDI_CC_ASSIGN_ADDR,4
	);



}

uint8_t read_dualmatrix_mode(void){
	return(eeprom_read_byte((const uint8_t *)(EEPROM_GLOBALS_ADDR+5)));
}

void read_globals(void){

	eeprom_read_block(
		(void *)&(s_matrix),(const void *)0,512
	);	//512 = 32bits or 4 Bytes per matrix * 16 banks * 8 steps
	
	knob_mode=eeprom_read_byte((const uint8_t *)(EEPROM_GLOBALS_ADDR));
			if (knob_mode>3) knob_mode=0;

	sub_knob_mode=eeprom_read_byte((const uint8_t *)(EEPROM_GLOBALS_ADDR+1));
			if (sub_knob_mode>4) sub_knob_mode=0;

	path_mode=eeprom_read_byte((const uint8_t *)(EEPROM_GLOBALS_ADDR+2));
			if (path_mode>3) path_mode=0;

	knob_layout_mode=eeprom_read_byte((const uint8_t *)(EEPROM_GLOBALS_ADDR+3));
			if (knob_layout_mode>3) knob_layout_mode=0;

	midi_mode=eeprom_read_byte((const uint8_t *)(EEPROM_GLOBALS_ADDR+4));
			if (midi_mode>5) midi_mode=0;
}

void read_lengths(void){
	uint8_t i;

	eeprom_read_block(
		(void *)&(patch_play_start), (const void *)(EEPROM_LENGTH_ADDR),16);
	eeprom_read_block(
		(void *)&(patch_play_end), (const void *)(EEPROM_LENGTH_ADDR+16),16);

	for (i=0;i<16;i++){
		if (patch_play_start[i]>7) patch_play_start[i]=0;
		if (patch_play_end[i]>7) patch_play_end[i]=7;
	}
	bank_play_start=eeprom_read_byte((const uint8_t *)(EEPROM_LENGTH_ADDR+32));
		if (bank_play_start>15) bank_play_start=0;

	bank_play_end=eeprom_read_byte((const uint8_t *)(EEPROM_LENGTH_ADDR+33));
		if (bank_play_end>15) bank_play_end=0;
}



/*
void read_bpms(void){
	uint8_t i;

	eeprom_read_block(
		(void *)&(bpm),(const void *)(EEPROM_BPM_ADDR),16*2
	);	//bpm is an array of 16 2-byte integers

	for (i=0;i<16;i++){
		if (bpm[i]>1500) bpm[i]=240;
	}
}
*/
