#include <stdarg.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>

#include "bendmatrix2.h"
#include "globals.h"
#include "midi.h"
#include "uart.h"
#include "outputs.h"
#include "memory.h"


void midi_setup_menu(void){
	static uint8_t last_button=255;

	hold_menu_display=1;
	seven_seg_key=1;

	if ((assign_midi_mode==1) ||  (assign_midi_mode==2) ||  (assign_midi_mode==3)){

		if (last_button!=assign_midi_curbut){	//pressed a new button, so update assign_midi_value to the new button's setting
			last_button=assign_midi_curbut; 

			if (assign_midi_mode==1)
				assign_midi_value=assigned_midi_notes[assign_midi_curbut];
			else if (assign_midi_mode==2)
				assign_midi_value=CC_number[assign_midi_curbut];

			#ifdef MAP_CCS_TO_BUTTONS
			else if (assign_midi_mode==3)
				assign_midi_value=assigned_midi_CCbuttons[assign_midi_curbut];
			#endif

			new_assign_midi_value=assign_midi_value;
		}
		else

		if (new_assign_midi_value!=assign_midi_value) {		//Turned the rotary, or recv'd incoming MIDI
			assign_midi_value=new_assign_midi_value;

			if (assign_midi_mode==1)
				assigned_midi_notes[assign_midi_curbut]=assign_midi_value;
			else if (assign_midi_mode==2)
				CC_number[assign_midi_curbut]=assign_midi_value;
			#ifdef MAP_CCS_TO_BUTTONS
			else if (assign_midi_mode==3)
				assigned_midi_CCbuttons[assign_midi_curbut]=assign_midi_value;
			#endif
		}
		
		if (show_midi_menu_display){
			if (show_midi_menu_display++>DISPLAYHOLD) show_midi_menu_display=0;

			if (assign_midi_mode==1) string_to_string("NOT",seven_seg);
			if (assign_midi_mode==2) string_to_string("CC ",seven_seg);
			if (assign_midi_mode==3) string_to_string("CCB",seven_seg);
		} else {
			number_to_string(assign_midi_value,seven_seg);
		}
		seven_seg[3]=0b111;


	} else if (assign_midi_mode==99){
		save_midi_assigns();
		string_to_string("OK ",seven_seg);
		assign_midi_mode=0;
	}

}

void read_midi(void){

	static uint8_t midi_ptr=0;
	static uint8_t midi_buff[3];
	static uint8_t clocks_recvd=0;
	uint8_t midi_in;
	uint8_t noteoff,noteon,note,k;
	#ifdef MAP_CCS_TO_BUTTONS
		uint8_t CCon,CCoff;
	#endif

//	if ((midi_mode==1) || (midi_mode==2) || (midi_mode==4) || (midi_mode==5) ){
	if ((midi_mode!=3) && (midi_mode!=0)){
		//not SEND-only or OFF

		if (get_next_uart_data(&midi_in)) {
		
			if (midi_in == 0xFE) midi_in=0x00;  //ignore active sensing

			if (midi_mode!=4){ 	//mode 4 is NO-Clock mode
				if (midi_in == 0xF8) {
					short maxclocks=24;
					if (midi_mode==6) maxclocks=48;
					if (midi_mode==7) maxclocks=12;

					if (++clocks_recvd==maxclocks) {
						clocks_recvd=0;
						clockreceived=1;
						seven_seg[3]|=BIT(1);
					}  else if (clocks_recvd==6) seven_seg[3]&=~BIT(1);

				}
			}
			
			if (midi_mode!=5) { //mode 5 is Clock only 

				if (midi_in & BIT(7)) midi_ptr=0;	//if its a command then start reading in again

				switch (midi_ptr){
					case (0):
						if (midi_in & BIT(7)){
							midi_buff[0]=midi_in;
							midi_ptr=1;
						}
					break;
			
					case (1):
						midi_buff[1]=midi_in;
						midi_ptr=2;
					break;

					case (2):
						midi_buff[2]=midi_in;
						midi_ptr=3;
					break;

				}

				if (midi_ptr==3){
				

					// noteon is a non-zero velocity NOTEON command
					// noteoff is a NOTEOFF command or a zero velocity NOTEON command

					noteon=((midi_buff[0] == (MIDI_CHANNEL | NOTEON)) && midi_buff[2]);

					noteoff=((midi_buff[0] == (MIDI_CHANNEL | NOTEOFF)) 
							|| ((midi_buff[0] == (MIDI_CHANNEL | NOTEON)) && !midi_buff[2]));	



					if (noteon || noteoff){

						//update the assign_midi_value
						if (assign_midi_mode){
							new_assign_midi_value= midi_buff[1];
							//midimatrix[assign_midi_curbut>15] |= BIT(assign_midi_curbut & 15);
						}

						//Figure out if the note we hit is assigned to a button, and light it up
						for (k=0;k<32;k++){
							if (assigned_midi_notes[k]==midi_buff[1]) {
								note=k & 15;
								if (noteon) midimatrix[k>15] |= BIT(note);
								if (noteoff) midimatrix[k>15] &= ~BIT(note);
							}

						}

					}

					#ifdef MAP_CCS_TO_BUTTONS
						CCon=((midi_buff[0] == (MIDI_CHANNEL | MIDI_com_CC)) && midi_buff[2]==127);
						CCoff=((midi_buff[0] == (MIDI_CHANNEL | MIDI_com_CC)) && midi_buff[2]==0);


						if (CCon || CCoff){
							if (assign_midi_mode) {
								new_assign_midi_value=midi_buff[1];
							//	midimatrix[assign_midi_curbut>15] |= BIT(assign_midi_curbut & 15);
							}

							for (k=0;k<32;k++){
								if (assigned_midi_CCbuttons[k]==midi_buff[1]) {
									note=k & 15;
									if (CCon) midimatrix[k>15] |= BIT(note);
									if (CCoff) midimatrix[k>15] &= ~BIT(note);
								}
							}
						}
					#endif




					#ifdef RECMIDICC
						if(midi_buff[0] == (MIDI_CHANNEL | MIDI_com_CC)){

							if (assign_midi_mode==2) new_assign_midi_value=midi_buff[1];

							for (k=0;k<4;k++){
								if (midi_buff[1]==CC_number[k]){
									ain[k]=midi_buff[2]<<1;
									ainchanged=1;
								}
							}
						}
					#endif

				}
			}
		}
	} 
	
	if ((midi_mode==3) || (midi_mode==5)) { //mode 3 is OFF, mode 5 is Clock only
		midimatrix[0]=0;midimatrix[1]=0;
	}
}



void wait_for_UART(void){	while ( !(UCSRA & BIT(UDRE)) ) ;}

void send_one_MIDI_byte(short status){
	if ((midi_mode!=1) && (midi_mode!=3)){
			wait_for_UART();
		UDR=status;				//send status byte
	}
}
void send_MIDI_command(short status, short data1, short data2){
	if ((midi_mode!=1) && (midi_mode!=3)){
			wait_for_UART();
		UDR=status;				//send status byte
			wait_for_UART();
		UDR=data1;				//send data byte1
			wait_for_UART();	
		UDR=data2;				//send data byte2
	}
}

void send_MIDI_CC(uint8_t ccno, uint8_t val){
	if ((midi_mode!=1) && (midi_mode!=3)){
		send_MIDI_command(MIDI_com_CC | MIDI_CHANNEL, CC_number[ccno], val);
	}
}






/*
uint8_t MIDInote2matrix(uint8_t note, uint8_t *k){

inputs 7-bit note value (0..127)

outputs number 0..15 specifying the button pressed
also outputs k, which indicates left or right matrix



usage: m = MIDInote2matrix(note,&k); midimatrix[k] |= BIT(m)


	uint8_t scalednote;

	scalednote=(note-SCALE_OFFSET) & 0b00011111; //bottom 5 bits only (this makes midi input wrap around the button matrix

	*k=(scalednote & 0b00000100) >> 2;
	return ((
		(scalednote & 0b00011000) >> 1) |
		(scalednote & 0b00000011) 
	);

}


uint8_t matrix2MIDInote(uint8_t button, uint8_t k){

inputs button 0..15, and k 0..1 specifying which button is to be converted to a MIDI note

outputs a MIDI note 0..127

midinote is SCALE_OFFSET plus the 5 bit number: B2 K0 B3 B1 B0



	return (SCALE_OFFSET+ ( 
		((0b00000001 & k)<<2) | 
		((0b00001100 & button)<<1) | 
		(0b00000011 & button) 
	));

}

*/

