#include <stdarg.h>
#include <ctype.h>
#include <avr/io.h>

#include "bendmatrix2.h"
#include "globals.h"
#include "outputs.h"
#include "midi.h"
#include "memory.h"

uint8_t SEVENSEG[49] = {
	0b01111110,		//0
	0b00110000,		//1
	0b01101101,		//2
	0b01111001,		//3
	0b00110011,		//4
	0b01011011,		//5
	0b01011111,		//6
	0b01110000,		//7
	0b01111111,		//8
	0b01111011,		//9
	0b01110111,		//A
	0b00011111,		//b
	0b00001101,		//c
	0b00111101,		//d
	0b01001111,		//E
	0b01000111,		//F
	0b01011110,		//G
	0b00010111,		//h
	0b00010000,		//i
	0b00111000,		//J
	0b00101111,		//K
	0b00001110,		//L
	0b01010101,		//m is bar n
	0b00010101,		//n
	0b00011101,		//o
	0b01100111,		//P
	0b01110011,		//q
	0b00000101,		//r
	0b00011011,		//S is 5 w/o top
	0b00001111,		//t
	0b00011100,		//u
	0b00011100,		//v is u
	0b01011100,		//w is bar u
	0b00110111,		//X is capital H
	0b00111011,		//Y
	0b00101101,		//z is weird looking
	0b01100011,		//TOPO
	0b01000000,		//SPIN0 a
	0b00100000,		//SPIN1 b
	0b00000001,		//SPIN2 g
	0b00000100,		//SPIN3 e
	0b00001000,		//SPIN4 d
	0b00010000,		//SPIN5 c
	0b00000001,		//SPIN6 g
	0b00000010,		//SPIN7 f
	0b00000000,		//BLANK
	0b00000001,		//DASH
	0b01001001,		//HORIZ LINES
	0b00110110		//VERT LINES
};



void start_output_shiftreg(void){
	OSR &= ~BIT(OSR_ST); //lower the latch
	OSR &= ~BIT(OSR_SH); //get ready to shift data in
}

void end_output_shiftreg(void){
	OSR |= BIT(OSR_ST);		//set the latch to output all bits at once
}

void output_8bits_shift_reg(unsigned char the_bits){
	unsigned char i;
	i=0;
	while (i<8){
		if (the_bits & BIT(i)) OSR |= BIT(OSR_DS);
		else OSR&=~BIT(OSR_DS);

		OSR |= BIT(OSR_SH); 	//shift data
		OSR &= ~BIT(OSR_SH);
		i++;
	}

}


void number_to_string(uint16_t num, char d[4]){
	uint16_t orig_num;

	orig_num=num;
	if (num>1000) {
		num-=1000;
		d[3]|=BIT(0);
	}
	else d[3]&=~BIT(0);

//	else if (num==1000) num=999;
//	else if (num==0) num=1;

	d[0]=num/100;
	d[1]=(num - (d[0]*100))/10;
	d[2]=(num - (d[0]*100) - (d[1]*10));

	//Blank leading zeros
	if (orig_num<100) { 	d[0]=BLANK; 
		if (orig_num<10)	d[1]=BLANK;
	}
}

static inline char get_sevensegchar(char a){
	if ((a>=65) && (a<=90)) return(a-ALPHA);
	else if ((a>=48) && (a<=57)) return(a-NUM);
	else if (a==' ') return(BLANK);
	else if (a=='|') return(VERT);
	else if (a=='_') return(HORIZ);
	else if (a=='-') return(DASH);
	else if (a=='^') return(TOPO);
	else return(a);
}

void string_to_string(char a[3], char d[4]){
	d[0]=get_sevensegchar(a[0]);
	d[1]=get_sevensegchar(a[1]);
	d[2]=get_sevensegchar(a[2]);

//	if (a[0]==' ') d[0]=BLANK; else d[0]=a[0]-ALPHA;
//	if (a[1]==' ') d[1]=BLANK; else d[1]=a[1]-ALPHA;
//	if (a[2]==' ') d[2]=BLANK; else d[2]=a[2]-ALPHA;
}



void set_BPMLED(char bpmvalue[4]){

// each time this function is called, display a different digit

	static uint8_t digit=0;
	short decimalplace;

	if (bpmvalue[3] & BIT(digit)) decimalplace=BIT(7);
	else decimalplace=0;

		// Send character data
		// pabcdefg
		// where p is the decimal place and a-g are the segments of the 7 SEG LED display
	output_8bits_shift_reg(decimalplace | SEVENSEG[(short)bpmvalue[digit]]);

		// Send select bit for digit 1, 2, or 3
		// 00000001 or 00000010 or 00000100 respectively
		// Note: the top 5 bits are unused, for future expansion!
	output_8bits_shift_reg(BIT(digit));
	
	if (++digit>2) digit=0;		
}




void output_shiftreg(unsigned short seq_leds, uint16_t matrixleds[2], char bpmvalue[4]){
//74HC595: uC to SEQ to MX0-7 to MX8-15 to BPMLED to matrix2 MX0-7 to matrix2 MX8-15
	
	start_output_shiftreg();

if (MZ==2){
	output_8bits_shift_reg(matrixleds[1] >> 8);			//matrix[1] MX24-31
	output_8bits_shift_reg(matrixleds[0] >> 8);			//matrix[0] MX8-15
}

	set_BPMLED(bpmvalue);

	output_8bits_shift_reg(matrixleds[1] & 0xFF);		//matrix[1] MX16-23
	output_8bits_shift_reg(matrixleds[0] & 0xFF);		//matrix[0] MX0-7
	
	output_8bits_shift_reg(seq_leds);					//Sequencer

	end_output_shiftreg();

}



void set_switches(uint16_t matrixsw[2]) {

/*
	SWITCH is the output port interfacing with the MT8816 chip
	SWITCH bit:
	0 strobe
	1 data
	2 AY0
	3 AY1
	4 AX0
	5 AY2
	6 RESET
	7 AX1 

	SWITCH_HI bit:
	SWITCH_HI_AX2 (6) is AX2

	matrix[0] connects X0-X3 (AX2=0) with Y0-Y3 (AY2=0)
	matrix[1] connects X4-X7 (AX2=1) with Y4-Y7 (AY2=1)

*/
	uint8_t i=0,k;
	static uint16_t oldmatrixsw[2]={0,0};
		
	for (k=0;k<2;k++){
		for(i=0;i<MZ*8;i++){
			if ((oldmatrixsw[k] & BIT(i)) != (matrixsw[k] & BIT(i))){

				//address
				SWITCH &= 0b01000011;			//blanks out the 6 address bits
				SWITCH_HI &= ~BIT(SW_AX2);		

				if (k) {
					SWITCH |= BIT(SW_AY2);
					//turning on AY2 enables Y4-Y7 for green matrix


					if (DUALMATRIXMODE && (MZ==2))	
						SWITCH_HI |= BIT(SW_AX2);
						//turning on AX2 enables X4-X7, which is 2x4x4
						//otherwise we have X0-X3 instead, which is a  4x8

					if (DUALMATRIXMODE && (MZ==1))
						SWITCH |= BIT(SW_AX1);
						//turning on AX1 enables X2-X3, which is 2x2x4
						//otherwise we have X0-X1 instead, which is a 2x8
				}

				if (i & BIT(0)) SWITCH |= BIT(SW_AY0);
				if (i & BIT(1)) SWITCH |= BIT(SW_AY1);
				if (i & BIT(2)) SWITCH |= BIT(SW_AX0);
				if (i & BIT(3)) SWITCH |= BIT(SW_AX1);

			
				//strobe high
				SWITCH |= BIT(SW_STROBE); 


				//data
				if (matrixsw[k] & BIT(i)) SWITCH |= BIT(SW_DATA);
				else SWITCH &= ~BIT(SW_DATA);

				//strobe low
				SWITCH &= ~BIT(SW_STROBE); 

				//send MIDI data
				if (matrixsw[k] & BIT(i)){

					send_MIDI_command(NOTEON | MIDI_CHANNEL, assigned_midi_notes[i+k*16], 0x7F);

					#ifdef MAP_CCS_TO_BUTTONS
						send_MIDI_command(MIDI_com_CC | MIDI_CHANNEL,  assigned_midi_CCbuttons[i+k*16] ,0x7F);
					#endif
				}else{

					send_MIDI_command(NOTEOFF | MIDI_CHANNEL,  assigned_midi_notes[i+k*16], 0x00);

					#ifdef MAP_CCS_TO_BUTTONS
						send_MIDI_command(MIDI_com_CC | MIDI_CHANNEL,  assigned_midi_CCbuttons[i+k*16] ,0x00);
					#endif
				}
			}

		}	

		oldmatrixsw[k]=matrixsw[k];
	}

	SWITCH &= ~BIT(SW_DATA);
}



void flash_seqs(uint8_t highlight){
	static uint8_t c_seq_flash=0;

	if (c_seq_flash++>(FLASHTIME*8)) c_seq_flash=1;

	if (c_seq_flash<(FLASHTIME*4))
		seq_leds=0xFF;

	else if ((c_seq_flash>(FLASHTIME*6)) && (c_seq_flash<(FLASHTIME*7)) && (highlight<8)) 
		seq_leds=BIT(highlight);

	else
		seq_leds=0x00;

}


void flash_save(void){
	static uint8_t c_save_flash=0;

	if (c_save_flash++>(FLASHTIME*8)) c_save_flash=1;

	if (c_save_flash<(FLASHTIME*4))
		PLAYSAVE_LED |= BIT(SAVE_LED);
	else
		PLAYSAVE_LED &= ~BIT(SAVE_LED);
}


void flash_matrix(uint16_t output_matrix[2]){
	static uint8_t c_bank_flash=0;

	if (c_bank_flash++>(FLASHTIME*8)) c_bank_flash=1;

	if (c_bank_flash<(FLASHTIME*4)) {
		output_matrix[0]=(uint16_t)(-1);
		output_matrix[1]=(uint16_t)(-1);
	} else {
		output_matrix[0]=0;
		output_matrix[1]=0;
	}		


}

void flash_one_bank(uint16_t output_matrix[2], uint8_t highlight){
	static uint8_t c_bank_flash=0;

	
	if (c_bank_flash++>(FLASHTIME*4)) c_bank_flash=1;

	if (c_bank_flash<(FLASHTIME)){

		if (MZ==1)	output_matrix[(highlight>7)?1:0]|=BIT(highlight & 7);
		else 		output_matrix[(highlight>15)?1:0]|=BIT(highlight & 15);

	}else{

		if (MZ==1)	output_matrix[(highlight>7)?1:0]&=~BIT(highlight & 7);
		else		output_matrix[(highlight>15)?1:0]&=~BIT(highlight & 15);

	}
}

void flash_matrix_on_matrix(uint16_t output_matrix[2],uint16_t flash_matrix[2],short brightness){
	static uint8_t c_bank_flash=0;

	if (c_bank_flash++>(FLASHTIME*4)) c_bank_flash=1;

	if (c_bank_flash<(FLASHTIME*brightness)) {
		output_matrix[0]=output_matrix[0] | flash_matrix[0];
		output_matrix[1]=output_matrix[1] | flash_matrix[1];
	}		
}

void flash_matrix_off_matrix(uint16_t output_matrix[2],uint16_t flash_matrix[2],short brightness){
	static uint8_t c_bank_flash=0;

	if (c_bank_flash++>(FLASHTIME*4)) c_bank_flash=1;

	if (c_bank_flash<(FLASHTIME*brightness)) {
		output_matrix[0]=output_matrix[0] & ~flash_matrix[0];
		output_matrix[1]=output_matrix[1] & ~flash_matrix[1];
	}		
}
