#include <avr/io.h>
#include <avr/interrupt.h>

#include "uart.h"

#define UARTBUFFSIZE 64
uint8_t uart_buff[UARTBUFFSIZE];
uint8_t uart_start_ptr=0;
uint8_t uart_end_ptr=0;

void init_uart(uint8_t ubbr){
	UBRRL=ubbr;
	UCSRA=0x00;
	UCSRB=(1<<TXEN) | (1<<RXEN) | (1<<RXCIE); 	//enable TX and RX and interrupt for RX
}


ISR(USART_RXC_vect){
	uint8_t udr=UDR;
	uint8_t ucsra=UCSRA;
	uint8_t uart_end_ptr_tmp=uart_end_ptr;

//calculate the next pointer position
	if (++uart_end_ptr_tmp>UARTBUFFSIZE) uart_end_ptr_tmp=0;

//return if the buffer is full
	if (uart_start_ptr==uart_end_ptr_tmp)
		return;

//return if there's a frame error, data overrun, or parity error
	if(ucsra & ((1<<FE)|(1<<DOR)|(1<<PE)))	
		return;

//store the incoming data in the buffer
	uart_buff[uart_end_ptr]=udr;

//update the pointer to the next position
	uart_end_ptr=uart_end_ptr_tmp;
}

uint8_t get_next_uart_data(uint8_t *data){
	
//make sure the buffer is not empty
	if (uart_start_ptr==uart_end_ptr)
		return (0);
	
//fetch the next byte and store it
	*data = uart_buff[uart_start_ptr];
	
//increment the uart buffer pointer
	if (++uart_start_ptr>UARTBUFFSIZE) uart_start_ptr=0;

	return(1);
}
